<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\Product;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Select;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Zend_Db;
use Zend_Db_Expr;

/**
 * Class PriceProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class PriceProvider implements EntityProviderInterface
{
    /** @var null */
    protected $ids;
    /** @var int */
    protected $website;
    /** @var array */
    protected $items = [];
    /** @var ResourceConnection */
    private $resourceConnection;

    /**
     * PriceProvider constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return mixed
     */
    public function current()
    {
        return current($this->items);
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    /**
     * @return bool|float|int|mixed|string|null
     */
    public function key()
    {
        return current($this->items)['id'];
    }

    /**
     * @return bool|mixed
     */
    public function valid()
    {
        return current($this->items);
    }

    /**
     *
     */
    public function rewind()
    {
        $connection = $this->resourceConnection->getConnection();
        $priceTable = $connection->getTableName('catalog_product_index_price');

        $select = $connection->select();
        $select->from(
            $priceTable,
            [
                'id'             => 'entity_id',
                'website_id'     => 'website_id',
                'final_price'    => new Zend_Db_Expr('IF (final_price,final_price,min_price)'),
                'customer_group' => 'customer_group_id',
                'min_price'      => 'min_price',
                'max_price'      => 'max_price',
                'base_price'     => new Zend_Db_Expr('IF (price,price,min_price)')
            ]
        );

        if (!empty($this->ids)) {
            $select->where('entity_id IN (' . implode(',', $this->ids) . ')');
        }

        if ($this->website !== null) {
            $select->where("website_id = {$this->website}");
        } else {
            $select->where("website_id = 0");
        }

        $select->order(['entity_id DESC', 'customer_group_id ASC']);

        $prices = $this->resourceConnection->getConnection()->fetchAll($select, [], Zend_Db::FETCH_ASSOC);

        foreach ($prices as $price) {
            $productId = $price['id'];

            if (!isset($this->items[$productId])) {
                $this->items[$productId] = new Product();
                $this->items[$productId]->setId($productId);
            }

            $finalPrice = (float)$price['final_price'];

            if (((float)$price['final_price'] > (float)$price['min_price'] && (float)$price['min_price'] != 0.0)) {
                $finalPrice = (float)$price['min_price'];
            }

            $this->items[$productId]->setData("pricing_{$price['customer_group']}", [
                'final_price' => $finalPrice,
                'max_price'   => (float)$price['max_price'],
                'min_price'   => (float)$price['min_price'],
                'base_price'  => (float)$price['base_price']
            ]);
        }
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->website = $store->getWebsiteId();
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        $this->website = $website->getId();
        return $this;
    }

    /**
     * @param $stockId
     * @return Select
     */
    protected function getSelectForStock($stockId)
    {
        return $this->resourceConnection->getConnection()
            ->select()
            ->from(
                [
                    's' => "inventory_stock_{$stockId}"
                ],
                [
                    'id'         => 'cpe.entity_id',
                    'quantity'   => 'quantity',
                    'is_salable' => 'is_salable',
                    'backorders' => 'csi.backorders'
                ]
            )->join(
                [
                    'cpe' => 'catalog_product_entity'
                ],
                's.sku = cpe.sku',
                []
            )->join(
                ['csi' => 'cataloginventory_stock_item'],
                ' cpe.entity_id = csi.product_id',
                ['backorders' => 'backorders']
            );
    }
}
